from matplotlib import pyplot as plt
from typing import Union
import math

from imshowtools.helper_functions import _convert_mode, _SUPPORTED_MODES, _imshow_finally, _RETURN_IMAGE_TYPES


def imshow(*images, cmap: str = 'viridis', rows: int = None, columns: int = None, mode: Union[str, list] = None,
           window_title: str = None, title: Union[str, list] = None, return_image: Union[bool, str] = False) -> None:
    """
    Shows image loaded by opencv after inverting the order of channels
    Can also be used to show single layer depth image
    Args:
        *images: one of more np.array of shape h,w,c or simple h,w
        cmap: specify a cmap to apply to all images (viridis by default)
        mode: specify a mode or color space one in RGB or BGR
        rows: number of rows to show
        columns: numbers of columns to show
        window_title: window title (not applicable for ipynb notebooks)
        title: title for the image, or list of titles, one for each image
        return_image: if one of ['RGB', 'RGBA', 'ARGB', 'BW', 'L', "BGR", "BGRA", "ABGR"] returns Image.
                      if True returns 'RGB'. Does not display image if set. if False, returns None, but displays image.
    Returns:
        None if return_image is False, else numpy.ndarray of shape [h,w,c] depending on its value.
    """
    plt.rcParams['image.cmap'] = cmap

    # Setting fig in other cases works,
    # But matplotlib will print a warning
    # <Figure size 432x288 with 0 Axes>
    fig = None
    if window_title is not None or return_image is True or return_image in _RETURN_IMAGE_TYPES:
        fig = plt.figure(window_title)

    title_list = None
    if type(title) is str:
        plt.title(title)
    elif type(title) is list:
        if len(title) == len(images) and all([type(el) == str for el in title]):
            title_list = title
        else:
            raise ValueError('Title can either be a string or a list of strings')

    mode_list = None
    if type(mode) is list:
        if len(mode) == len(images) and all([type(el) == str or el is None for el in mode]):
            mode_list = mode
        else:
            raise ValueError(f'Mode can either be a string or a list of strings from {_SUPPORTED_MODES}')

    no_of_images = len(images)
    if no_of_images is 0:
        print("Please provide at least one image to display! Try again")
        return

    if no_of_images is 1:
        img = images[0]
        img = _convert_mode(img, mode)
        plt.imshow(img)
        plt.axis('off')
        return _imshow_finally(fig, return_image)

    if rows is None:
        rows = int(math.sqrt(no_of_images))
    if columns is None:
        columns = int(math.ceil(no_of_images / rows))

    fig, axes = plt.subplots(rows, columns)
    for index, axis in enumerate(axes.reshape(-1)):
        if index < no_of_images:
            img = images[index]
            if mode_list:
                img = _convert_mode(img, mode_list[index], index)
            else:
                img = _convert_mode(img, mode, index)
            if title_list:
                axis.set_title(title_list[index])
            axis.imshow(img)
        axis.axis('off')

    return _imshow_finally(fig, return_image)


def cvshow(*images, cmap: str = 'viridis', rows: int = None, columns: int = None, window_title: str = None,
           title: Union[str, list] = None, return_image: Union[bool, str] = False) -> None:
    """
    Convenience function for displaying images loaded by OpenCV which are read as BGR by default,
    same as using imshow with `mode='BGR'`
    Args:
        *images: one of more np.array of shape h,w,c or simple h,w
        cmap: specify a cmap to apply to all images (viridis by default)
        rows: number of rows to show
        columns: numbers of columns to show
        window_title: window title (not applicable for ipynb notebooks)
        title: title for the image, or list of titles - one for each image
        return_image: if one of ['RGB', 'RGBA', 'ARGB', 'BW', 'L', "BGR", "BGRA", "ABGR"] returns Image.
                      if True returns 'RGB'. Does not display image if set. if False, returns None, but displays image.
    Returns:
        None if return_image is False, else numpy.ndarray of shape [h,w,c] depending on its value.
    """
    return imshow(*images, cmap=cmap, rows=rows, columns=columns, mode='BGR',
                  window_title=window_title, title=title, return_image=return_image)



import pathlib
from setuptools import setup
from imshowtools import __version__

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="imshowtools",
    version=__version__,
    description="imshowtools contains simplified imshow functions to show multiple images and with other options",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/saravanabalagi/imshowtools",
    author="Saravanabalagi Ramachandran",
    author_email="saravanabalagi@hotmail.com",
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["imshowtools"],
    include_package_data=True,
    install_requires=["matplotlib", "numpy"]
)



import warnings

import numpy as np
from matplotlib import pyplot as plt

_SUPPORTED_MODES = ['RGB', 'BGR']
_RETURN_IMAGE_TYPES = ['RGB', 'RGBA', 'ARGB', 'BW', 'L', "BGR", "BGRA", "ABGR"]


def _has_one_channel(img):
    if len(img.shape) == 2:
        return True
    if len(img.shape) == 3 and img.shape[2] == 1:
        return True
    return False


def _has_three_or_four_channels(img):
    if len(img.shape) == 3 and img.shape[2] in [3, 4]:
        return True
    return False


def _convert_mode(img, mode=None, index=None):
    if mode is None:
        mode = 'RGB'

    if img is None:
        raise ValueError(f'Image{(" " + index) if index else ""} is None')

    mode = mode.upper()
    if mode not in _SUPPORTED_MODES:
        raise ValueError('Mode {} not found. Use one from {}'.format(mode, _SUPPORTED_MODES))

    if _has_one_channel(img):
        # squeeze if it has shape [h, w, 1]
        if len(img.shape) == 3:
            img = img[:, :, 0]

        # Colorspace conversion not required
        # for single channel images
        mode = 'RGB'

    if mode == 'RGB':
        return img
    elif mode == 'BGR':
        if not _has_three_or_four_channels(img):
            warnings.warn(f'Image {index if index else ""} has {img.shape[2]} channels, expecting 3 or 4 channels.\n'
                          f'Using BGR mode may not produce expected output as it simply reverses channel order.')
        return img[:, :, ::-1]


def _imshow_finally(fig, return_image):
    if return_image is None or return_image is False:
        plt.show()
        return
    else:
        return_type = "RGB"
        if type(return_image) is bool and return_image is True:
            return_type = "RGBA"
        elif type(return_image) is str and return_image in _RETURN_IMAGE_TYPES:
            return_type = return_image
        else:
            warnings.warn(f'`return_image` attribute has to either bool or one of {_RETURN_IMAGE_TYPES}\n'
                          f'Given return image: {return_image} of type {type(return_image).__name__}\n'
                          f'Returning the image with default type: {return_type}')

        fig.canvas.draw()
        width, height = fig.get_size_inches() * fig.get_dpi()
        image_argb = np.frombuffer(fig.canvas.tostring_argb(), dtype='uint8').reshape(int(height), int(width), 4)
        image_rgba = image_argb[:, :, [1, 2, 3, 0]]       # convert from argb to rgba
        plt.close(fig)

        if return_type == "RGB":
            return image_rgba[..., :3]
        elif return_type == "BGR":
            return image_rgba[..., [2, 1, 0]]
        elif return_type == "BGRA":
            return image_rgba[..., [2, 1, 0, 3]]
        elif return_type == "ABGR":
            return image_rgba[..., [3, 2, 1, 0]]
        elif return_type in ["BW", "L"]:
            # ITU-R 601-2 luma transform
            return np.dot(image_rgba[..., :3], [0.2989, 0.5870, 0.1140])
        elif return_type == "ARGB":
            return image_argb

        return image_rgba
